package com.skyversation.poiaddr.util;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.skyversation.poiaddr.entity.AddressV4Result;
import com.skyversation.poiaddr.service.AreaService;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

/***
 * 接口返回消息封装管理工具
 */

@Slf4j
public class MessageManage {

    private static MessageManage messageManage = new MessageManage();
    private MessageManage(){}
    public static MessageManage getInstance(){
        if(messageManage == null) {
            messageManage = new MessageManage();
        }
        return messageManage;
    }

    public String getResultContent(int code, String content, String message){
        JSONObject json = new JSONObject();
        try {
            json.put("code", code);
            if(content == null || content.equals("")){
                json.put("content", null);
            } else {
                json.put("content", content);
            }
            json.put("message", message);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        log.info(json.toString());
        return json.toString();
    }

    public String getResultContent(int code, AddressV4Result addV4, String message){
        JSONObject json = new JSONObject();
        try {
            json.put("code", code);
            if(addV4 == null){
                json.put("content", null);
            } else {
                JSONObject content = new JSONObject();
                content.put("resultCode", addV4.getResultCode());
                content.put("message", addV4.getMessage());
                content.put("exactResult", addV4.getExactResult());
                content.put("recomResult", addV4.getRecomResult());
                json.put("content", content);
            }
            json.put("message", message);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return JSON.toJSONString(json, SerializerFeature.DisableCircularReferenceDetect);
    }

    public String getAddressResultContent(int code, Object bean, String message){
        JSONObject json = new JSONObject();
        try {
            json.put("code", code);
            if(bean == null){
                json.put("content", null);
            } else {
                JSONObject content = JSON.parseObject(JSON.toJSONString(bean));
                JSONArray pois = content.getJSONArray("data");
                for(int i = 0; i < pois.size(); i++){
                    JSONObject poi = pois.getJSONObject(i);
                    poi.put("parent",new JSONArray());
                    poi.put("address",poi.getString("address")==null?"":poi.getString("address"));
                    poi.put("distance",new JSONArray());
                    poi.put("pname",poi.getString("pname")==null?"":poi.getString("pname"));
                    poi.put("importance",new JSONArray());
                    JSONObject biz_ext = new JSONObject();
                    biz_ext.put("cost",new JSONArray());
                    biz_ext.put("rating",new JSONArray());
                    poi.put("biz_ext",biz_ext);
                    poi.put("biz_type",new JSONArray());
                    String oCityname = poi.getString("cityname");
                    String oAdname = poi.getString("adname");
                    poi.put("cityname",poi.get("pname"));
                    //网格信息太复杂放最后
                    poi.put("type",poi.getString("type")==null?"":poi.getString("type"));
                    poi.put("parking_type","");
                    poi.put("photos",new JSONArray());
                    poi.put("typecode","");
                    poi.put("shopinfo","");
                    poi.put("poiweight",new JSONArray());
                    poi.put("childtype",new JSONArray());
                    poi.put("adname",oCityname);
                    poi.put("name",poi.getString("name")==null?"":poi.getString("name"));
                    //村居信息太复杂放最后
                    //微网格信息太复杂放最后
                    poi.put("location",poi.getString("location")==null?"":poi.getString("location"));
                    poi.put("tel",new JSONArray());
                    poi.put("shopid",new JSONArray());
                    poi.put("id","");
                    double x = poi.getDouble("lon");
                    double y = poi.getDouble("lat");
                    AreaService areaService = AreaService.getInstance();
                    //TODO 网格信息
                    poi.put("网格信息",areaService.isInGridPolygon(x,y).getProperties());
                    //TODO 村居信息
                    poi.put("村居信息",areaService.isInCJPolygon(x,y).getProperties());
                    //TODO 微网格信息
                    poi.put("微网格信息",areaService.isInWGridPolygon(x,y).getProperties());

                }


                content.put("pois", pois);
                content.remove("data");


                JSONObject suggestion = new JSONObject();
                suggestion.put("keywords",new JSONArray());
                suggestion.put("cities",new JSONArray());
                content.put("suggestion",suggestion);
                content.put("count","");
                content.put("infocode","");
                content.put("status","");
                content.put("info","");


                json.put("content", JSON.toJSON(content));

            }
            json.put("message", message);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        log.info(json.toString());
        return json.toString();
    }

    public String getResultContent(int code, List lis, String message){
        JSONObject json = new JSONObject();
        try {
            json.put("code", code);
            json.put("message", message);
            if(lis != null && lis.size() > 0){
                JSONArray data = new JSONArray();
                for(Object obj : lis){
                    data.add(JSON.toJSON(obj));
                }
                json.put("content", data);
            } else {
                json.put("content", null);
            }

        } catch (JSONException e) {
            e.printStackTrace();
        }
        log.info(json.toString());
        return json.toString();
    }
    public String getResultContent(int code, List lis, String message, Long total){
        JSONObject json = new JSONObject();
        try {
            json.put("code", code);
            json.put("message", message);
            if(total == null || total < 0){
                json.put("total", 0);
            } else {
                json.put("total", total);
            }
            if(lis != null && lis.size() > 0){
                JSONArray data = new JSONArray();
                for(Object obj : lis){
                    data.add(JSON.toJSON(obj));
                }
                json.put("content", data);
            } else {
                json.put("content", "");
            }

        } catch (JSONException e) {
            e.printStackTrace();
        }
        log.info(json.toString());
        return json.toString();
    }


}
