package com.skyversation.poiaddr.util.geotools;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import java.math.BigDecimal;

/**
 * 坐标系转换工具
 */
public class GeoHelpScript {
    /**
     * 常量定义
     */
    private final Double PI = Math.PI;
    private final Double x_PI = (PI * 3000.0) / 180.0;
    private final Double ee = 0.00669342162296594323;
    private final Double a = 6378245.0;

    /**
     * 主运行方法
     * geoJson格式坐标系转换
     * @param fileDataStr  geoJson字符串
     * @param selectValue  转换前的坐标系名称 {"WGS84（国际通用）", "GCJ02（高德、QQ地图）", "BD09（百度地图）", "上海2000坐标系"}
     * @param selectValue2 转换后的坐标系名称 {"WGS84（国际通用）", "GCJ02（高德、QQ地图）", "BD09（百度地图）", "上海2000坐标系"}
     * @return JSONObject
     * 转换前的坐标系名称和转换后的坐标系名称不能一样
     */
    public JSONObject run(String fileDataStr, String selectValue, String selectValue2) {
        JSONObject fileData = JSONObject.parseObject(fileDataStr);
            JSONArray features = fileData.getJSONArray("features");
            for (int i = 0; i < features.size(); i++) {
                JSONObject feature = (JSONObject) features.get(i);
                JSONObject geometry = feature.getJSONObject("geometry");
                JSONArray coordinates = geometry.getJSONArray("coordinates");
                CoordinateSystemTransformation(coordinates, selectValue, selectValue2);
            }
        return fileData;
    }

    /**
     * shp格式的坐标系转换
     * @param fileDataStr shp文件数据
     * @param selectValue 转换前坐标系 {"WGS84（国际通用）", "GCJ02（高德、QQ地图）", "BD09（百度地图）", "上海2000坐标系"}
     * @param selectValue2 转换后坐标系 {"WGS84（国际通用）", "GCJ02（高德、QQ地图）", "BD09（百度地图）", "上海2000坐标系"}
     * @return JSONArray
     */
    public JSONArray runShp(String fileDataStr, String selectValue, String selectValue2) {
        JSONArray fileData = JSONArray.parseArray(fileDataStr);
        for (int i = 0; i < fileData.size(); i++) {
            JSONObject feature = fileData.getJSONObject(i);
            if(feature.containsKey("fileName") && feature.containsKey("dataList")){
                //      如果是geoJson的zip包格式
                JSONArray dataList = feature.getJSONArray("dataList");
                for (int j = 0; j < dataList.size(); j++) {
                    JSONObject feature2 = dataList.getJSONObject(j);
                    JSONObject geometry = feature2.getJSONObject("geometry");
                    JSONArray coordinates = geometry.getJSONArray("coordinates");
//                  坐标系转换
                    CoordinateSystemTransformation(coordinates, selectValue, selectValue2);
                }
            }else{
                JSONObject geometry = feature.getJSONObject("geometry");
                JSONArray coordinates = geometry.getJSONArray("coordinates");
//                坐标系转换
                CoordinateSystemTransformation(coordinates, selectValue, selectValue2);
            }
        }
        return fileData;
    }

    /**
     * 迭代遍历 coordinates并转换坐标
     *
     * @param coordinates
     */
    public void CoordinateSystemTransformation(JSONArray coordinates, String selectValue, String selectValue2) {
        if (coordinates.get(0) instanceof BigDecimal) {
            Double[] outData;
            if ("WGS84".equals(selectValue)) {
                switch (selectValue2) {
                    case "GCJ02":
                        outData = wgs84_to_gcj02(coordinates.getDouble(0), coordinates.getDouble(1));
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                    case "BD09":
                        outData = wgs84_to_bd09(coordinates.getDouble(0), coordinates.getDouble(1));
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                    case "SH2000":
                        outData = wgs84_to_shcj(coordinates.getDouble(0), coordinates.getDouble(1));
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                }
            } else if ("GCJ02".equals(selectValue)) {
                switch (selectValue2) {
                    case "WGS84":
                        outData = gcj02_to_wgs84(coordinates.getDouble(0), coordinates.getDouble(1));
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                    case "BD09":
                        outData = gcj02_to_bd09(coordinates.getDouble(0), coordinates.getDouble(1));
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                    case "SH2000":
                        outData = gcj02_to_shcj(coordinates.getDouble(0), coordinates.getDouble(1));
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                }
            } else if ("BD09".equals(selectValue)) {
                switch (selectValue2) {
                    case "WGS84":
                        outData = bd09_to_wgs84(coordinates.getDouble(0), coordinates.getDouble(1));
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                    case "GCJ02":
                        outData = bd09_to_gcj02(coordinates.getDouble(0), coordinates.getDouble(1));
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                    case "SH2000":
                        Double[] lngLat = bd09_to_wgs84(coordinates.getDouble(0), coordinates.getDouble(1));
                        outData = wgs84_to_shcj(lngLat[0], lngLat[1]);
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                }
            } else if ("SH2000".equals(selectValue)) {
                switch (selectValue2) {
                    case "WGS84":
                        outData = shcj_to_wgs84(coordinates.getDouble(0), coordinates.getDouble(1));
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                    case "BD09":
                        Double[] lngLat = shcj_to_wgs84(coordinates.getDouble(0), coordinates.getDouble(1));
                        outData = wgs84_to_bd09(lngLat[0], lngLat[1]);
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                    case "GCJ02":
                        outData = shcj_to_gcj02(coordinates.getDouble(0), coordinates.getDouble(1));
                        coordinates.set(0, outData[0]);
                        coordinates.set(1, outData[1]);
                        break;
                }
            }
        } else {
            for (int i = 0; i < coordinates.size(); i++) {
                JSONArray coordinate = coordinates.getJSONArray(i);
                CoordinateSystemTransformation(coordinate, selectValue, selectValue2);
            }
        }
    }

    public Double[] wgs84_to_gcj02(Double lng, Double lat) {
        if (out_of_china(lng, lat)) {
            return new Double[]{lng, lat};
        } else {
            double dlat = transformlat(lng - 105.0, lat - 35.0);
            double dlng = transformlng(lng - 105.0, lat - 35.0);
            double radlat = (lat / 180.0) * PI;
            double magic = Math.sin(radlat);
            magic = 1 - ee * magic * magic;
            double sqrtmagic = Math.sqrt(magic);
            dlat = (dlat * 180.0) / (((a * (1 - ee)) / (magic * sqrtmagic)) * PI);
            dlng = (dlng * 180.0) / ((a / sqrtmagic) * Math.cos(radlat) * PI);
            Double[] gg = new Double[2];
            gg[0] = lng + dlng;
            gg[1] = lat + dlat;
            return gg;
        }
    }

    public Double[] gcj02_to_wgs84(Double lng, Double lat) {
        if (out_of_china(lng, lat)) {
            return new Double[]{lng, lat};
        } else {
            double dlat = transformlat(lng - 105.0, lat - 35.0);
            double dlng = transformlng(lng - 105.0, lat - 35.0);
            double radlat = (lat / 180.0) * PI;
            double magic = Math.sin(radlat);
            magic = 1 - ee * magic * magic;
            double sqrtmagic = Math.sqrt(magic);
            dlat = (dlat * 180.0) / (((a * (1 - ee)) / (magic * sqrtmagic)) * PI);
            dlng = (dlng * 180.0) / ((a / sqrtmagic) * Math.cos(radlat) * PI);
            Double[] gg = new Double[2];
            gg[0] = lng - dlng;
            gg[1] = lat - dlat;
            return gg;
        }
    }


    public Double[] wgs84_to_shcj(Double x, Double y) {
        Double[] xy = shcj_get_UTM_from_WGS(x, y);
        return utm_to_shcj4(xy[0], xy[1]);
    }

    public Double[] shcj_to_wgs84(Double x, Double y) {
        Double[] xy = shcj_to_utm4(x, y);
        return shcj_get_WGS_from_UTM(xy[0], xy[1]);
    }

    public Double[] wgs84_to_bd09(Double x, Double y) {
        Double[] xy = wgs84_to_gcj02(x, y);
        return gcj02_to_bd09(xy[0], xy[1]);
    }

    public Double[] bd09_to_wgs84(Double x, Double y) {
        Double[] xy = bd09_to_gcj02(x, y);
        return gcj02_to_wgs84(xy[0], xy[1]);
    }

    public Double[] gcj02_to_shcj(Double x, Double y) {
        Double[] xy = gcj02_to_wgs84(x, y);
        return wgs84_to_shcj(xy[0], xy[1]);
    }

    public Double[] shcj_to_gcj02(Double x, Double y) {
        Double[] xy = shcj_to_wgs84(x, y);
        return wgs84_to_gcj02(xy[0], xy[1]);
    }

    public Double[] shcj_to_utm4(Double x, Double y) {
        double DX, DY, T, K;
        DX = 499999.90104;
        DY = 3455696.403019;
        T = -0.0000001755;
        K = 0.999600149344;
        return covert_by_four_parm(x, y, DX, DY, T, K);
    }

    public Double[] utm_to_shcj4(Double x, Double y) {
        double DX, DY, T, K;
        DX = -500199.29965;
        DY = -3457078.805985;
        T = 0.0000001755;
        K = 1.0004000106;
        return covert_by_four_parm(x, y, DX, DY, T, K);
    }

    public Double[] bd09_to_gcj02(Double bd_lon, Double bd_lat) {
        Double x = bd_lon - 0.0065;
        Double y = bd_lat - 0.006;
        double z = Math.sqrt(x * x + y * y) - 0.00002 * Math.sin(y * x_PI);
        double theta = Math.atan2(y, x) - 0.000003 * Math.cos(x * x_PI);
        double gg_lng = z * Math.cos(theta);
        double gg_lat = z * Math.sin(theta);
        Double[] gg = new Double[2];
        gg[0] = gg_lng;
        gg[1] = gg_lat;
        return gg;
    }

    public Double[] gcj02_to_bd09(Double lng, Double lat) {
        double z = Math.sqrt(lng * lng + lat * lat) + 0.00002 * Math.sin(lat * x_PI);
        double theta = Math.atan2(lat, lng) + 0.000003 * Math.cos(lng * x_PI);
        Double[] gg = new Double[2];
        gg[0] = z * Math.cos(theta) + 0.0065;
        gg[1] = z * Math.sin(theta) + 0.006;
        return gg;
    }

    public Boolean out_of_china(Double lng, Double lat) {
        // 纬度3.86~53.55,经度73.66~135.05
        return !(lng > 73.66 && lng < 135.05 && lat > 3.86 && lat < 53.55);
    }


    public double transformlat(Double lng, Double lat) {
        double ret = -100.0 + 2.0 * lng + 3.0 * lat + 0.2 * lat * lat + 0.1 * lng * lat + 0.2 * Math.sqrt(Math.abs(lng));
        ret += ((20.0 * Math.sin(6.0 * lng * PI) + 20.0 * Math.sin(2.0 * lng * PI)) * 2.0) / 3.0;
        ret += ((20.0 * Math.sin(lat * PI) + 40.0 * Math.sin((lat / 3.0) * PI)) * 2.0) / 3.0;
        ret += ((160.0 * Math.sin((lat / 12.0) * PI) + 320 * Math.sin((lat * PI) / 30.0)) * 2.0) / 3.0;
        return ret;
    }

    public double transformlng(Double lng, Double lat) {
        double ret = 300.0 + lng + 2.0 * lat + 0.1 * lng * lng + 0.1 * lng * lat +
                0.1 * Math.sqrt(Math.abs(lng));
        ret += ((20.0 * Math.sin(6.0 * lng * PI) + 20.0 * Math.sin(2.0 * lng * PI)) * 2.0) / 3.0;
        ret += ((20.0 * Math.sin(lng * PI) + 40.0 * Math.sin((lng / 3.0) * PI)) * 2.0) / 3.0;
        ret += ((150.0 * Math.sin((lng / 12.0) * PI) + 300.0 * Math.sin((lng / 30.0) * PI)) * 2.0) / 3.0;
        return ret;
    }

    //四参数公式
    public Double[] covert_by_four_parm(Double x, Double y, Double dx, Double dy, Double a, Double k) {
        double px = 0;
        double py = 0;
        px = x * k * Math.cos(a) - y * k * Math.sin(a) + dx;
        py = x * k * Math.sin(a) + y * k * Math.cos(a) + dy;
        Double[] gg = new Double[2];
        gg[0] = px;
        gg[1] = py;
        return gg;
    }

    public Double[] shcj_get_UTM_from_WGS(Double lon, Double lat) {
        double a = 6378137;
        double b = 6356752.3142451;
        double f = (a - b) / a;

        double eSquare = 2 * f - f * f;
        double k0 = 0.9996;
        double lonOrigin = 121.46714714;
        double FN = 0;
        // # 确保longtitude位于-180.00----179.9之间
        double lonTemp = lon + 180 - Math.floor((lon + 180) / 360) * 360 - 180;
        double latRad = (lat * PI) / 180;
        double lonRad = (lonTemp * PI) / 180;
        double lonOriginRad = (lonOrigin * PI) / 180;
        double e2Square = eSquare / (1 - eSquare);

        double V = a / Math.sqrt(1 - eSquare * Math.pow(Math.sin(latRad), 2));
        double T = Math.pow(Math.tan(latRad), 2);
        double C = e2Square * Math.pow(Math.cos(latRad), 2);
        double A = Math.cos(latRad) * (lonRad - lonOriginRad);
        double M = a * ((1 - eSquare / 4 - (3 * Math.pow(eSquare, 2)) / 64 - (5 * Math.pow(eSquare, 3)) / 256) * latRad - ((3 * eSquare) / 8 + (3 * Math.pow(eSquare, 2)) / 32 + (45 * Math.pow(eSquare, 3)) / 1024) * Math.sin(2 * latRad) + ((15 * Math.pow(eSquare, 2)) / 256 + (45 * Math.pow(eSquare, 3)) / 1024) * Math.sin(4 * latRad) - ((35 * Math.pow(eSquare, 3)) / 3072) * Math.sin(6 * latRad));

        // # x
        double UTMEasting = k0 * V * (A + ((1 - T + C) * Math.pow(A, 3)) / 6 + ((5 - 18 * T + Math.pow(T, 2) + 72 * C - 58 * e2Square) * Math.pow(A, 5)) / 120) + 500000.0;
        //  # y
        double UTMNorthing = k0 * (M + V * Math.tan(latRad) * (Math.pow(A, 2) / 2 + ((5 - T + 9 * C + 4 * Math.pow(C, 2)) * Math.pow(A, 4)) / 24 + ((61 - 58 * T + Math.pow(T, 2) + 600 * C - 330 * e2Square) * Math.pow(A, 6)) / 720));
        //# 南半球纬度起点为10000000.0m
        UTMNorthing += FN;
        Double[] gg = new Double[2];
        gg[0] = UTMEasting;
        gg[1] = UTMNorthing;
        return gg;
    }


    public Double[] shcj_get_WGS_from_UTM(Double x, Double y) {
        //WGS84
        double a = 6378137; //椭球体长半轴
        double b = 6356752.3142451; //椭球体短半轴
        x = 500000 - x;
        double k0 = 0.9996;
        double e = Math.sqrt(1 - Math.pow(b, 2) / Math.pow(a, 2));
        // # calculate the meridional arc
        double M = y / k0;
        //# calculate footprint latitude
        double mu = M / (a * (1 - Math.pow(e, 2) / 4 - (3 * Math.pow(e, 4)) / 64 - (5 * Math.pow(e, 6)) / 256));
        double e1 = (1 - Math.pow(1 - Math.pow(e, 2), 1.0 / 2)) / (1 + Math.pow(1 - Math.pow(e, 2), 1.0 / 2));

        double J1 = (3 * e1) / 2 - (27 * Math.pow(e1, 3)) / 32;
        double J2 = (21 * Math.pow(e1, 2)) / 16 - (55 * Math.pow(e1, 4)) / 32;
        double J3 = (151 * Math.pow(e1, 3)) / 96;
        double J4 = (1097 * Math.pow(e1, 4)) / 512;
        double fp = mu + J1 * Math.sin(2 * mu) + J2 * Math.sin(4 * mu) + J3 * Math.sin(6 * mu) + J4 * Math.sin(8 * mu);

        // # Calculate Latitude and Longitude

        double e2 = Math.pow(e, 2) / (1 - Math.pow(e, 2));
        double C1 = e2 * Math.pow(Math.cos(fp), 2);
        double T1 = Math.pow(Math.tan(fp), 2);
        double R1 = (a * (1 - Math.pow(e, 2))) / Math.pow(1 - Math.pow(e * Math.sin(fp), 2), 3.0 / 2);
        //# This is the same as rho in the forward conversion formulas above, but calculated for fp instead of lat.
        double N1 = a / Math.pow(1 - Math.pow(e * Math.sin(fp), 2), 1.0 / 2);
        //# This is the same as nu in the forward conversion formulas above, but calculated for fp instead of lat.
        double D = x / (N1 * k0);

        double Q1 = (N1 * Math.tan(fp)) / R1;
        double Q2 = Math.pow(D, 2) / 2;
        double Q3 = ((5 + 3 * T1 + 10 * C1 - 4 * Math.pow(C1, 2) - 9 * e2) * Math.pow(D, 4)) / 24;
        double Q4 = ((61 + 90 * T1 + 298 * C1 + 45 * Math.pow(T1, 2) - 3 * Math.pow(C1, 2) - 252 * e2) * Math.pow(D, 6)) / 720;
        double lat = ((fp - Q1 * (Q2 - Q3 + Q4)) * 180) / PI;
        // System.out.println("lat===="+Math.toRadians(fp - Q1*(Q2 - Q3 + Q4)));
        double Q5 = D;
        double Q6 = ((1 + 2 * T1 + C1) * Math.pow(D, 3)) / 6;
        double Q7 = ((5 - 2 * C1 + 28 * T1 - 3 * Math.pow(C1, 2) + 8 * e2 + 24 * Math.pow(T1, 2)) * Math.pow(D, 5)) / 120;
        double lonmid = 121.46714714;
        double lon = lonmid - (((Q5 - Q6 + Q7) / Math.cos(fp)) * 180) / PI;
        Double[] gg = new Double[2];
        gg[0] = lon;
        gg[1] = lat;
        return gg;
    }
}
