package com.skyversation.poiaddr.util.geotools;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.json.simple.parser.JSONParser;

import java.io.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 根据街道名称和村名得到青浦匹配的村庄geojson合集
 */
public class GeoJsonIntersector {
    /*public static void main(String[] args) throws Exception {
        run("香花桥街道", "向阳村,爱星村,大联村,东方村,金米村,东斜村,新姚村,新桥村,民惠第三居委会,燕南村,泾阳村,胜利村,天一村,曹泾村,香花桥居委会,朝阳村,金星村,都汇华庭居委会,陈桥村,盈中村,石西村,杨元村,袁家村,七汇村,郏一村,青山居委会,金巷居委会", "博文中学");
//        run("香花桥街道", "燕南村,泾阳村,东方村,大联村,金米村,新桥村,爱星村,大盈居委会,民惠居委会,民惠第二居委会,民惠第三居委会,新姚村,友爱居委会,胜利村,天一村,向阳村,曹泾村,东斜村", "博文小学");
//        run("香花桥街道","香花桥居委会,朝阳村,金星村,都汇华庭居委会,盈中村,石西村,杨元村,七汇村,袁家村,郏一村,青山居委会,金巷居委会","香花桥小学");
//        run("夏阳街道", "青华居委,南箐园社区居委,塔湾村,南门塔湾,大生桥,李腰泾,王仙村,城南村,塘郁村,枫泾村,太来村,新阳村,金家村,夏阳村,夏阳湖社区居委,青平居委,华骥苑社区居委,桂花园社区居委", "豫英小学");
//        run("夏阳街道", "南门塔湾,大生桥,李腰泾,王仙村,城南村,塘郁村,枫泾村,太来村,新阳村,金家村,夏阳村,丝绸新村,志诚上和里", "豫才中学");
//        run("盈浦街道", "南横村", "思源中学");
//        run("夏阳街道","仓桥村","仓桥村");

    }*/

    public static void run(String jdNameStr, String czNameStr, String fileName) throws Exception {
        List<String> jdNameList = new ArrayList<>(Arrays.asList(jdNameStr.split(",")));
        List<String> nameList = new ArrayList<>();
        if (czNameStr != null && !czNameStr.isEmpty()) {
            nameList = new ArrayList<>(Arrays.asList(czNameStr.split(",")));
            System.out.println("******需要匹配的村名个数：" + nameList.size());
        }

        JSONObject processingGeoJson = readGeoJson("src/main/resources/geojson/qp_cunju.geojson");
        JSONArray features = (JSONArray) processingGeoJson.get("features");
        JSONArray newFeatures = new JSONArray();
        for (Object feature : features) {
            JSONObject item = JSONObject.parseObject(feature.toString());
            String jiezhen = item.getJSONObject("properties").getString("jz");
            String cunName = item.getJSONObject("properties").getString("cjname");
            if (jdNameList.contains(jiezhen)) {
                if (czNameStr == null || czNameStr.isEmpty()) {
                    newFeatures.add(feature);
                } else if (nameList.contains(cunName)) {
                    newFeatures.add(feature);
                    System.out.println("++++匹配到:" + jiezhen + "." + cunName);
                    nameList.remove(cunName);
                }
            }
        }
        if (nameList.size() > 0) {
            for (String cName : nameList) {
                System.out.println("----没有匹配到：" + cName);
            }
        }
        System.out.println("******匹配到的村名个数：" + newFeatures.size());
        System.out.println("******没有匹配到的村名个数：" + nameList.size());
        processingGeoJson.put("features", newFeatures);
        // 5. 保存结果
        saveGeoJson(processingGeoJson, "output/result_" + fileName + ".geojson");
        System.out.println("解析完成,已保存结果geojson文件到output/,文件名称：result_" + fileName + ".geojson");
    }

    private static JSONObject readGeoJson(String path) throws Exception {
        String fileData = new JSONParser().parse(new FileReader(path)).toString();
        return JSONObject.parseObject(fileData);
    }

    private static void saveGeoJson(JSONObject geoJson, String path) throws Exception {
        try (FileWriter writer = new FileWriter(path)) {
            writer.write(geoJson.toJSONString());
        }
    }
}