package com.skyversation.poiaddr.util.net;
import java.net.URI;
import java.net.URLEncoder;
//import java.net.http.HttpClient;
//import java.net.http.HttpRequest;
//import java.net.http.HttpResponse;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.*;

/**
 * java 11支持的版本
 * executeAsyncRequests 方法：
 * 接收请求的 URL 列表、请求头和请求参数作为输入。
 * 创建一个固定大小的线程池，线程池的大小与 URL 列表的大小相同。
 * 使用 CountDownLatch 来确保所有任务都执行完毕。
 * 遍历 URL 列表，为每个 URL 创建一个 RequestTask 任务，并提交到线程池中执行。
 * 等待所有任务完成后，关闭线程池并返回所有请求的结果。
 * RequestTask 类：
 * 实现了 Runnable 接口，用于定义请求任务。
 * 在 run 方法中，构建请求 URL，添加请求头和参数，使用 HttpClient 发送异步请求。
 * 处理响应并将结果添加到 results 列表中。
 * 在 finally 块中调用 latch.countDown() 方法，减少计数器的值，表示任务完成。
 * saveResultsToDatabase 方法：
 * 模拟将请求结果存储到数据库的操作，这里只是简单地打印结果。
 *
 * 注意事项
 * 线程池大小：线程池的大小应根据系统资源和请求的性质进行合理调整，避免创建过多线程导致系统资源耗尽。
 * 异常处理：在 RequestTask 的 run 方法中捕获并处理可能的异常，确保任务的稳定性。
 * 数据库存储：实际应用中，需要根据具体的数据库类型和使用的数据库访问框架（如 JDBC、MyBatis 等）来实现将结果存储到数据库的功能。
 * 通过这种方式，可以实现多线程异步请求，并在所有请求完成后将结果存储到数据库。
 */
public class MultiThreadedAsyncHttpClient {
//
//    public static void main(String[] args) {
//        // 模拟请求的 URL 列表
//        List<String> urls = Arrays.asList(
//                "https://www.example.com/api/endpoint1",
//                "https://www.example.com/api/endpoint2"
//        );
//
//        // 配置请求头
//        Map<String, String> headers = new HashMap<>();
//        headers.put("Authorization", "Bearer your_token");
//        headers.put("Content-Type", "application/json");
//
//        // 配置请求参数
//        Map<String, String> params = new HashMap<>();
//        params.put("param1", "value1");
//        params.put("param2", "value2");
//
//        // 执行多线程异步请求
//        List<String> results = executeAsyncRequests(urls, headers, params);
//
//        // 模拟将结果存储到数据库
//        saveResultsToDatabase(results);
//    }
//
//    public static List<String> executeAsyncRequests(List<String> urls, Map<String, String> headers, Map<String, String> params) {
//        // 创建一个固定大小的线程池
//        int threadPoolSize = urls.size();
//        ExecutorService executorService = Executors.newFixedThreadPool(threadPoolSize);
//
//        // 创建 CountDownLatch 用于等待所有任务完成
//        CountDownLatch latch = new CountDownLatch(threadPoolSize);
//
//        // 用于存储所有请求的结果
//        List<String> results = new CopyOnWriteArrayList<>();
//
//        // 创建并提交任务到线程池
//        for (String url : urls) {
//            executorService.submit(new RequestTask(url, headers, params, latch, results));
//        }
//
//        try {
//            // 等待所有任务完成
//            latch.await();
//        } catch (InterruptedException e) {
//            Thread.currentThread().interrupt();
//            e.printStackTrace();
//        }
//
//        // 关闭线程池
//        executorService.shutdown();
//
//        return results;
//    }
//
//    static class RequestTask implements Runnable {
//        private final String url;
//        private final Map<String, String> headers;
//        private final Map<String, String> params;
//        private final CountDownLatch latch;
//        private final List<String> results;
//
//        public RequestTask(String url, Map<String, String> headers, Map<String, String> params, CountDownLatch latch, List<String> results) {
//            this.url = url;
//            this.headers = headers;
//            this.params = params;
//            this.latch = latch;
//            this.results = results;
//        }
//
//        @Override
//        public void run() {
//            try {
//                // 创建 HttpClient 实例
//                HttpClient client = HttpClient.newHttpClient();
//
//                // 构建请求 URL，添加参数
//                StringBuilder query = new StringBuilder();
//                for (Map.Entry<String, String> entry : params.entrySet()) {
//                    if (query.length() > 0) {
//                        query.append("&");
//                    }
//                    query.append(URLEncoder.encode(entry.getKey(), StandardCharsets.UTF_8));
//                    query.append("=");
//                    query.append(URLEncoder.encode(entry.getValue(), StandardCharsets.UTF_8));
//                }
//                String fullUrl = url + "?" + query.toString();
//
//                // 创建 HttpRequest 实例
//                HttpRequest.Builder requestBuilder = HttpRequest.newBuilder()
//                        .uri(URI.create(fullUrl));
//
//                // 添加请求头
//                for (Map.Entry<String, String> entry : headers.entrySet()) {
//                    requestBuilder.header(entry.getKey(), entry.getValue());
//                }
//
//                HttpRequest request = requestBuilder.build();
//
//                // 发送异步请求
//                CompletableFuture<HttpResponse<String>> future = client.sendAsync(request, HttpResponse.BodyHandlers.ofString());
//                HttpResponse<String> response = future.get();
//
//                // 处理响应
//                results.add(response.body());
//            } catch (Exception e) {
//                e.printStackTrace();
//            } finally {
//                // 任务完成，减少计数器
//                latch.countDown();
//            }
//        }
//    }
//
//    public static void saveResultsToDatabase(List<String> results) {
//        // 模拟将结果存储到数据库
//        System.out.println("将以下结果存储到数据库：");
//        for (String result : results) {
//            System.out.println(result);
//        }
//    }
}