package com.skyversation.poiaddr.util.test;

import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;

/**
 * 根据测试数据条数，测试市大数据中心接口响应时间
 */
public class NetworkRequestTest {
    private static final String URL = "https://service-api.onemap.sh.gov.cn/data-service-manage-service/MapProxyApi/eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJhcHBsaWNhdGlvbl9pZCI6NjEsImFwcGxpY2F0aW9uX25hbWUiOiLpnZLmtabkuozkuInnu7TmnI3liqHns7vnu58iLCJleHAiOjIwNDY2Nzg0MDN9.IKUMdjUX4U1jncIUNren-iotL7duXI90aLECMjpvUX8/address_search/MapServer?region=310118&page_num=1&page_size=1&query=馨浦苑西区"; // 替换为实际的接口地址
    private static final int TEST_DURATION = 1000; // 测试请求次数

    public static void main(String[] args) throws InterruptedException {
        int threadCount = Runtime.getRuntime().availableProcessors();
        ExecutorService executorService = Executors.newFixedThreadPool(threadCount);

        long startTime = System.currentTimeMillis();
        List<Future<?>> futures = new ArrayList<>();
        for (int i = 0; i < TEST_DURATION - 1; i++) {
            futures.add(executorService.submit(() -> {
                try (CloseableHttpClient httpClient = HttpClients.createDefault()) {
                    HttpGet httpGet = new HttpGet(URL);
                    try (CloseableHttpResponse response = httpClient.execute(httpGet)) {
                        // 处理响应
                        System.out.println("-----------------请求结果：" + response.toString());
                        EntityUtils.consume(response.getEntity());
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }));
        }

        // 等待所有任务完成
        for (Future<?> future : futures) {
            try {
                future.get();
            } catch (InterruptedException | ExecutionException e) {
                e.printStackTrace();
            }
        }
        // 关闭线程池
        executorService.shutdown();
        long endTime = System.currentTimeMillis();
        System.out.println("线程个数：" + threadCount);
        long time = (endTime - startTime) / 1000;
        System.out.println(TEST_DURATION + "个请求所用的时间: " + time + "秒！");
        System.out.println("平均一秒请求" + TEST_DURATION / time + "个！");
    }
}